@@ -36,21 +36,31 @@
     html_logo_url = "https://raw.githubusercontent.com/smol-rs/smol/master/assets/images/logo_fullsize_transparent.png"
 )]
 
+#[cfg(not(feature = "portable-atomic"))]
 extern crate alloc;
 
 use core::fmt;
 use core::future::Future;
 use core::marker::PhantomPinned;
 use core::pin::Pin;
-use core::sync::atomic::{AtomicUsize, Ordering};
 use core::task::{Context, Poll};
-use core::usize;
 
+#[cfg(not(feature = "portable-atomic"))]
 use alloc::sync::Arc;
+#[cfg(not(feature = "portable-atomic"))]
+use core::sync::atomic::{AtomicUsize, Ordering};
 
-use concurrent_queue::{ConcurrentQueue, PopError, PushError};
-use event_listener::{Event, EventListener};
-use event_listener_strategy::{easy_wrapper, EventListenerFuture, Strategy};
+#[cfg(feature = "portable-atomic")]
+use portable_atomic::{AtomicUsize, Ordering};
+#[cfg(feature = "portable-atomic")]
+use portable_atomic_util::Arc;
+
+use concurrent_queue::{ConcurrentQueue, ForcePushError, PopError, PushError};
+use event_listener_strategy::{
+    easy_wrapper,
+    event_listener::{Event, EventListener},
+    EventListenerFuture, Strategy,
+};
 use futures_core::ready;
 use futures_core::stream::Stream;
 use pin_project_lite::pin_project;
@@ -68,6 +78,9 @@ struct Channel<T> {
     /// Stream operations while the channel is empty and not closed.
     stream_ops: Event,
 
+    /// Closed operations while the channel is not closed.
+    closed_ops: Event,
+
     /// The number of currently active `Sender`s.
     sender_count: AtomicUsize,
 
@@ -87,6 +100,7 @@ impl<T> Channel<T> {
             // Notify all receive and stream operations.
             self.recv_ops.notify(usize::MAX);
             self.stream_ops.notify(usize::MAX);
+            self.closed_ops.notify(usize::MAX);
 
             true
         } else {
@@ -126,6 +140,7 @@ pub fn bounded<T>(cap: usize) -> (Sender<T>, Receiver<T>) {
         send_ops: Event::new(),
         recv_ops: Event::new(),
         stream_ops: Event::new(),
+        closed_ops: Event::new(),
         sender_count: AtomicUsize::new(1),
         receiver_count: AtomicUsize::new(1),
     });
@@ -167,6 +182,7 @@ pub fn unbounded<T>() -> (Sender<T>, Receiver<T>) {
         send_ops: Event::new(),
         recv_ops: Event::new(),
         stream_ops: Event::new(),
+        closed_ops: Event::new(),
         sender_count: AtomicUsize::new(1),
         receiver_count: AtomicUsize::new(1),
     });
@@ -256,6 +272,29 @@ impl<T> Sender<T> {
         })
     }
 
+    /// Completes when all receivers have dropped.
+    ///
+    /// This allows the producers to get notified when interest in the produced values is canceled and immediately stop doing work.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// # futures_lite::future::block_on(async {
+    /// use async_channel::{unbounded, SendError};
+    ///
+    /// let (s, r) = unbounded::<i32>();
+    /// drop(r);
+    /// s.closed().await;
+    /// # });
+    /// ```
+    pub fn closed(&self) -> Closed<'_, T> {
+        Closed::_new(ClosedInner {
+            sender: self,
+            listener: None,
+            _pin: PhantomPinned,
+        })
+    }
+
     /// Sends a message into this channel using the blocking strategy.
     ///
     /// If the channel is full, this method will block until there is room.
@@ -286,6 +325,47 @@ impl<T> Sender<T> {
         self.send(msg).wait()
     }
 
+    /// Forcefully push a message into this channel.
+    ///
+    /// If the channel is full, this method will replace an existing message in the
+    /// channel and return it as `Ok(Some(value))`. If the channel is closed, this
+    /// method will return an error.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// # futures_lite::future::block_on(async {
+    /// use async_channel::{bounded, SendError};
+    ///
+    /// let (s, r) = bounded(3);
+    ///
+    /// assert_eq!(s.send(1).await, Ok(()));
+    /// assert_eq!(s.send(2).await, Ok(()));
+    /// assert_eq!(s.force_send(3), Ok(None));
+    /// assert_eq!(s.force_send(4), Ok(Some(1)));
+    ///
+    /// assert_eq!(r.recv().await, Ok(2));
+    /// assert_eq!(r.recv().await, Ok(3));
+    /// assert_eq!(r.recv().await, Ok(4));
+    /// # });
+    /// ```
+    pub fn force_send(&self, msg: T) -> Result<Option<T>, SendError<T>> {
+        match self.channel.queue.force_push(msg) {
+            Ok(backlog) => {
+                // Notify a blocked receive operation. If the notified operation gets canceled,
+                // it will notify another blocked receive operation.
+                self.channel.recv_ops.notify_additional(1);
+
+                // Notify all blocked streams.
+                self.channel.stream_ops.notify(usize::MAX);
+
+                Ok(backlog)
+            }
+
+            Err(ForcePushError(reject)) => Err(SendError(reject)),
+        }
+    }
+
     /// Closes the channel.
     ///
     /// Returns `true` if this call has closed the channel and it was not closed already.
@@ -450,6 +530,24 @@ impl<T> Sender<T> {
             channel: self.channel.clone(),
         }
     }
+
+    /// Returns whether the senders belong to the same channel.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// # futures_lite::future::block_on(async {
+    /// use async_channel::unbounded;
+    ///
+    /// let (s, r) = unbounded::<()>();
+    /// let s2 = s.clone();
+    ///
+    /// assert!(s.same_channel(&s2));
+    /// # });
+    /// ```
+    pub fn same_channel(&self, other: &Sender<T>) -> bool {
+        Arc::ptr_eq(&self.channel, &other.channel)
+    }
 }
 
 impl<T> Drop for Sender<T> {
@@ -589,7 +687,7 @@ impl<T> Receiver<T> {
     /// # Blocking
     ///
     /// Rather than using asynchronous waiting, like the [`recv`](Self::recv) method,
-    /// this method will block the current thread until the message is sent.
+    /// this method will block the current thread until the message is received.
     ///
     /// This method should not be used in an asynchronous context. It is intended
     /// to be used such that a channel can be used in both asynchronous and synchronous contexts.
@@ -777,6 +875,24 @@ impl<T> Receiver<T> {
             channel: self.channel.clone(),
         }
     }
+
+    /// Returns whether the receivers belong to the same channel.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// # futures_lite::future::block_on(async {
+    /// use async_channel::unbounded;
+    ///
+    /// let (s, r) = unbounded::<()>();
+    /// let r2 = r.clone();
+    ///
+    /// assert!(r.same_channel(&r2));
+    /// # });
+    /// ```
+    pub fn same_channel(&self, other: &Receiver<T>) -> bool {
+        Arc::ptr_eq(&self.channel, &other.channel)
+    }
 }
 
 impl<T> fmt::Debug for Receiver<T> {
@@ -853,7 +969,7 @@ impl<T> futures_core::stream::FusedStream for Receiver<T> {
     }
 }
 
-/// A [`Sender`] that prevents the channel from not being closed.
+/// A [`Sender`] that does not prevent the channel from being closed.
 ///
 /// This is created through the [`Sender::downgrade`] method. In order to use it, it needs
 /// to be upgraded into a [`Sender`] through the `upgrade` method.
@@ -899,7 +1015,7 @@ impl<T> fmt::Debug for WeakSender<T> {
     }
 }
 
-/// A [`Receiver`] that prevents the channel from not being closed.
+/// A [`Receiver`] that does not prevent the channel from being closed.
 ///
 /// This is created through the [`Receiver::downgrade`] method. In order to use it, it needs
 /// to be upgraded into a [`Receiver`] through the `upgrade` method.
@@ -1115,7 +1231,7 @@ pin_project! {
     }
 }
 
-impl<'a, T> EventListenerFuture for SendInner<'a, T> {
+impl<T> EventListenerFuture for SendInner<'_, T> {
     type Output = Result<(), SendError<T>>;
 
     /// Run this future with the given `Strategy`.
@@ -1171,7 +1287,7 @@ pin_project! {
     }
 }
 
-impl<'a, T> EventListenerFuture for RecvInner<'a, T> {
+impl<T> EventListenerFuture for RecvInner<'_, T> {
     type Output = Result<T, RecvError>;
 
     /// Run this future with the given `Strategy`.
@@ -1201,6 +1317,59 @@ impl<'a, T> EventListenerFuture for RecvInner<'a, T> {
     }
 }
 
+easy_wrapper! {
+    /// A future returned by [`Sender::closed()`].
+    #[derive(Debug)]
+    #[must_use = "futures do nothing unless you `.await` or poll them"]
+    pub struct Closed<'a, T>(ClosedInner<'a, T> => ());
+    #[cfg(all(feature = "std", not(target_family = "wasm")))]
+    pub(crate) wait();
+}
+
+pin_project! {
+    #[derive(Debug)]
+    #[project(!Unpin)]
+    struct ClosedInner<'a, T> {
+        // Reference to the sender.
+        sender: &'a Sender<T>,
+
+        // Listener waiting on the channel.
+        listener: Option<EventListener>,
+
+        // Keeping this type `!Unpin` enables future optimizations.
+        #[pin]
+        _pin: PhantomPinned
+    }
+}
+
+impl<'a, T> EventListenerFuture for ClosedInner<'a, T> {
+    type Output = ();
+
+    /// Run this future with the given `Strategy`.
+    fn poll_with_strategy<'x, S: Strategy<'x>>(
+        self: Pin<&mut Self>,
+        strategy: &mut S,
+        cx: &mut S::Context,
+    ) -> Poll<()> {
+        let this = self.project();
+
+        loop {
+            // Check if the channel is closed.
+            if this.sender.is_closed() {
+                return Poll::Ready(());
+            }
+
+            // Not closed - now start listening for notifications or wait for one.
+            if this.listener.is_some() {
+                // Poll using the given strategy
+                ready!(S::poll(strategy, &mut *this.listener, cx));
+            } else {
+                *this.listener = Some(this.sender.channel.closed_ops.listen());
+            }
+        }
+    }
+}
+
 #[cfg(feature = "std")]
 use std::process::abort;
 
