@@ -10,7 +10,7 @@ impl PrintOperation {
     pub fn error(&self) -> Option<glib::Error> {
         unsafe {
             let mut error = std::ptr::null_mut();
-            ffi::gtk_print_operation_get_error(self.to_glib_none().0, &mut error);
+            crate::ffi::gtk_print_operation_get_error(self.to_glib_none().0, &mut error);
             if error.is_null() {
                 None
             } else {
