@@ -1,6 +1,9 @@
 use std::convert::TryFrom;
 
-use crate::{Literal, err::{InvalidToken, TokenKind}};
+use crate::{
+    err::{InvalidToken, TokenKind},
+    Literal,
+};
 
 
 /// Helper macro to call a `callback` macro four times for all combinations of
@@ -47,6 +50,7 @@ impl_specific_lit_to_lit!(crate::CharLit<B>, Char);
 impl_specific_lit_to_lit!(crate::StringLit<B>, String);
 impl_specific_lit_to_lit!(crate::ByteLit<B>, Byte);
 impl_specific_lit_to_lit!(crate::ByteStringLit<B>, ByteString);
+impl_specific_lit_to_lit!(crate::CStringLit<B>, CString);
 
 
 
@@ -69,7 +73,7 @@ macro_rules! impl_tt_to_lit {
     }
 }
 
-helper!(impl_tt_to_lit, );
+helper!(impl_tt_to_lit,);
 
 
 // ==============================================================================================
@@ -106,7 +110,7 @@ macro_rules! impl_tt_to_lit {
     }
 }
 
-helper!(impl_tt_to_lit, );
+helper!(impl_tt_to_lit,);
 
 
 // ==============================================================================================
@@ -122,6 +126,7 @@ fn kind_of(lit: &Literal<String>) -> TokenKind {
         Literal::Char(_) => TokenKind::CharLit,
         Literal::Byte(_) => TokenKind::ByteLit,
         Literal::ByteString(_) => TokenKind::ByteStringLit,
+        Literal::CString(_) => TokenKind::CStringLit,
     }
 }
 
@@ -173,6 +178,7 @@ helper!(impl_for_specific_lit, crate::CharLit<String>, Char, CharLit);
 helper!(impl_for_specific_lit, crate::StringLit<String>, String, StringLit);
 helper!(impl_for_specific_lit, crate::ByteLit<String>, Byte, ByteLit);
 helper!(impl_for_specific_lit, crate::ByteStringLit<String>, ByteString, ByteStringLit);
+helper!(impl_for_specific_lit, crate::CStringLit<String>, CString, CStringLit);
 
 
 // ==============================================================================================
@@ -205,6 +211,7 @@ helper_no_refs!(impl_specific_lit_to_pm_lit, CharLit, Char, CharLit);
 helper_no_refs!(impl_specific_lit_to_pm_lit, StringLit, String, StringLit);
 helper_no_refs!(impl_specific_lit_to_pm_lit, ByteLit, Byte, ByteLit);
 helper_no_refs!(impl_specific_lit_to_pm_lit, ByteStringLit, ByteString, ByteStringLit);
+helper_no_refs!(impl_specific_lit_to_pm_lit, CStringLit, CString, CStringLit);
 
 
 // ==============================================================================================
@@ -239,7 +246,7 @@ macro_rules! impl_from_tt_for_bool {
     };
 }
 
-helper!(impl_from_tt_for_bool, );
+helper!(impl_from_tt_for_bool,);
 
 // ==============================================================================================
 // ===== `From<BoolLit> for pm::Ident`
@@ -255,7 +262,7 @@ macro_rules! impl_bool_lit_to_pm_lit {
     };
 }
 
-helper_no_refs!(impl_bool_lit_to_pm_lit, );
+helper_no_refs!(impl_bool_lit_to_pm_lit,);
 
 
 mod tests {
@@ -278,6 +285,7 @@ mod tests {
     //! let _ = litrs::Literal::<String>::from(give::<litrs::StringLit<String>>());
     //! let _ = litrs::Literal::<String>::from(give::<litrs::ByteLit<String>>());
     //! let _ = litrs::Literal::<String>::from(give::<litrs::ByteStringLit<String>>());
+    //! let _ = litrs::Literal::<String>::from(give::<litrs::CStringLit<String>>());
     //!
     //! let _ = litrs::Literal::<&'static str>::from(give::<litrs::BoolLit>());
     //! let _ = litrs::Literal::<&'static str>::from(give::<litrs::IntegerLit<&'static str>>());
@@ -286,6 +294,7 @@ mod tests {
     //! let _ = litrs::Literal::<&'static str>::from(give::<litrs::StringLit<&'static str>>());
     //! let _ = litrs::Literal::<&'static str>::from(give::<litrs::ByteLit<&'static str>>());
     //! let _ = litrs::Literal::<&'static str>::from(give::<litrs::ByteStringLit<&'static str>>());
+    //! let _ = litrs::Literal::<&'static str>::from(give::<litrs::CStringLit<&'static str>>());
     //!
     //!
     //! let _ = litrs::Literal::from(give::<proc_macro::Literal>());
@@ -313,6 +322,9 @@ mod tests {
     //! let _ = litrs::ByteStringLit::try_from(give::<proc_macro::Literal>());
     //! let _ = litrs::ByteStringLit::try_from(give::<&proc_macro::Literal>());
     //!
+    //! let _ = litrs::CStringLit::try_from(give::<proc_macro::Literal>());
+    //! let _ = litrs::CStringLit::try_from(give::<&proc_macro::Literal>());
+    //!
     //!
     //! let _ = litrs::BoolLit::try_from(give::<proc_macro::TokenTree>());
     //! let _ = litrs::BoolLit::try_from(give::<&proc_macro::TokenTree>());
@@ -334,6 +346,9 @@ mod tests {
     //!
     //! let _ = litrs::ByteStringLit::try_from(give::<proc_macro::TokenTree>());
     //! let _ = litrs::ByteStringLit::try_from(give::<&proc_macro::TokenTree>());
+    //!
+    //! let _ = litrs::CStringLit::try_from(give::<proc_macro::TokenTree>());
+    //! let _ = litrs::CStringLit::try_from(give::<&proc_macro::TokenTree>());
     //! ```
 }
 
@@ -376,6 +391,9 @@ mod tests_proc_macro2 {
     //! let _ = litrs::ByteStringLit::try_from(give::<proc_macro2::Literal>());
     //! let _ = litrs::ByteStringLit::try_from(give::<&proc_macro2::Literal>());
     //!
+    //! let _ = litrs::CStringLit::try_from(give::<proc_macro2::Literal>());
+    //! let _ = litrs::CStringLit::try_from(give::<&proc_macro2::Literal>());
+    //!
     //!
     //! let _ = litrs::BoolLit::try_from(give::<proc_macro2::TokenTree>());
     //! let _ = litrs::BoolLit::try_from(give::<&proc_macro2::TokenTree>());
@@ -397,5 +415,8 @@ mod tests_proc_macro2 {
     //!
     //! let _ = litrs::ByteStringLit::try_from(give::<proc_macro2::TokenTree>());
     //! let _ = litrs::ByteStringLit::try_from(give::<&proc_macro2::TokenTree>());
+    //!
+    //! let _ = litrs::CStringLit::try_from(give::<proc_macro2::TokenTree>());
+    //! let _ = litrs::CStringLit::try_from(give::<&proc_macro2::TokenTree>());
     //! ```
 }
